Module: TypeToolkit::MethodPatch
- Included in:
- Method, UnboundMethod
- Defined in:
- lib/type_toolkit/method_patch.rb
Overview
@requires_ancestor: MethodOrUnboundMethod
Instance Method Summary collapse
-
#abstract? ⇒ Boolean
Returns true if this method is an abstract method that hasn't been implemented.
Instance Method Details
#abstract? ⇒ Boolean
Returns true if this method is an abstract method that hasn't been implemented.
Calling it will raise an AbstractMethodNotImplementedError.
10 11 12 13 14 |
# File 'lib/type_toolkit/method_patch.rb', line 10 def abstract? return false unless TypeToolkit::HasAbstractMethods === (owner = self.owner) owner.abstract_method?(name) end |